"use strict";
/* eslint-disable class-methods-use-this, fp/no-class, fp/no-this */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@graphql-tools/utils");
const graphql_1 = require("graphql");
const Logger_1 = __importDefault(require("../Logger"));
const log = Logger_1.default.child({
    namespace: 'LazyLoaderSchemaDirective',
});
const lazyLoaded = Symbol('LAZY_LOADED');
const wrapLazyLoadableField = (field, object) => {
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    const lazyLoad = object.lazyLoad;
    if (!lazyLoad) {
        return;
    }
    log.debug('decorating %s field on %s Object', field.name, object.name);
    const resolve = field.resolve || graphql_1.defaultFieldResolver;
    field.resolve = async (node, args, context, info) => {
        const currentValue = node[field.name];
        if (currentValue !== undefined) {
            return resolve(node, args, context, info);
        }
        if (!node[lazyLoaded]) {
            node[lazyLoaded] = lazyLoad(node, args, context, info);
        }
        const newNode = await node[lazyLoaded];
        return resolve(newNode, args, context, info);
    };
};
class LazyLoaderSchemaDirective extends utils_1.SchemaDirectiveVisitor {
    visitObject(object) {
        const fieldMap = object.getFields();
        for (const fieldName of Object.keys(fieldMap)) {
            wrapLazyLoadableField(fieldMap[fieldName], object);
        }
    }
}
exports.default = LazyLoaderSchemaDirective;
